package io.github.bymartrixx.vtd.gui;

import com.google.common.collect.Lists;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.mojang.blaze3d.systems.RenderSystem;
import io.github.bymartrixx.vtd.VTDMod;
import io.github.bymartrixx.vtd.gui.widget.*;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.logging.log4j.Level;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.*;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_287;
import net.minecraft.class_289;
import net.minecraft.class_290;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_3532;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_4587;
import net.minecraft.class_5253;

public class VTDScreen extends class_437 {
    private static VTDScreen instance;
    public final Map<String, List<String>> selectedPacks; // {"$category":["$pack","$pack"],"$category":["$pack"]}
    protected final class_2561 subtitle;
    private final PackNameTextFieldWidget.TooltipSupplier TOOLTIP_SUPPLIER = (textField, nameValidity, matrices, mouseX, mouseY) -> {
        class_2561 text = null;
        switch (nameValidity) {
            case RESERVED_WINDOWS:
                text = new class_2588("vtd.fileNameValidity.reservedWindows");
                break;
            case INVALID_WINDOWS:
                text = new class_2588("vtd.fileNameValidity.invalidWindows");
                break;
            case REGEX_DOESNT_MATCH:
                text = new class_2588("vtd.fileNameValidity.regexDoesntMatch", PackNameTextFieldWidget.fileNameRegex);
                break;
            case FILE_EXISTS:
                text = new class_2588("vtd.fileNameValidity.fileExists");
                break;
            case VALID:
            default:
                break;
        }

        if (text != null) {
            this.method_25424(matrices, text, textField.field_22760, textField.field_22761 - (textField.method_25364() / 2));
        }
    };
    private final class_437 previousScreen;
    private final ArrayList<class_4185> tabButtons = Lists.newArrayList();
    private class_4185 tabLeftButton;
    private class_4185 tabRightButton;
    private DownloadButtonWidget downloadButton;
    private PackListWidget listWidget;
    private SelectedPacksListWidget selectedPacksListWidget;
    private PackNameTextFieldWidget packNameField;
    private int tabIndex = 0;
    private int selectedTabIndex = 0;
    /**
     * The download progress. A percentage represented as a float.
     * The value {@code 1.0F} means that the download progress bar should not be rendered.
     * Use {@link #resetDownloadProgress()} to set the progress to {@code 1.0F}.
     */
    private float downloadProgress = -1.0F;

    public VTDScreen(class_437 previousScreen, class_2561 subtitle) {
        super(new class_2588("vtd.title"));
        this.previousScreen = previousScreen;
        this.selectedPacks = new LinkedHashMap<>();
        this.subtitle = subtitle;

        VTDScreen.instance = this;
    }

    public VTDScreen(class_437 previousScreen, class_2561 subtitle, Map<String, List<String>> selectedPacks) {
        super(new class_2588("vtd.title"));
        this.previousScreen = previousScreen;
        this.selectedPacks = selectedPacks;
        this.subtitle = subtitle;

        VTDScreen.instance = this;
    }

    /**
     * Get the number of "tabs" that should be generated/rendered.
     *
     * @param width the width of the screen.
     * @return The max number of "tabs"
     */
    private static int getTabNum(int width) {
        // 110 is the 3 buttons (20 * 3) and the margin between (10 between the buttons, 10 to the right of the right button and 10 * 2 for both sides of the screen)
        // The 130 is the width of the "tab" buttons and the margin (120 + 10)
        // The 1 is to make an "overflow" effect
        return (width - 110) / 130 + 1;
    }

    public static VTDScreen getInstance() {
        return instance;
    }

    private void download(DownloadButtonWidget button) {
        Thread downloadThread = new Thread(() -> {
            try {
                // Reverse this.selectedPacks order
                Map<String, List<String>> selectedPacks2 = new LinkedHashMap<>();
                ArrayList<String> keys = new ArrayList<>(this.selectedPacks.keySet());

                for (int i = keys.size() - 1; i >= 0; --i) {
                    String key = keys.get(i);
                    List<String> values = this.selectedPacks.get(key);

                    List<String> valuesReverse = new ArrayList<>();
                    for (int j = values.size() - 1; j >= 0; --j) {
                        valuesReverse.add(values.get(j));
                    }

                    selectedPacks2.put(key, valuesReverse);
                }

                JsonObject selectedPacks = VTDMod.GSON.toJsonTree(selectedPacks2).getAsJsonObject();
                this.downloadProgress = 0.0F;

                try (CloseableHttpClient client = HttpClients.createDefault()) {
                    // Get the download link
                    HttpPost httpPost = new HttpPost(VTDMod.BASE_URL + "/assets/server/zipresourcepacks.php");

                    List<NameValuePair> params = new ArrayList<>();
                    params.add(new BasicNameValuePair("version", "1.16"));
                    params.add(new BasicNameValuePair("packs", VTDMod.GSON.toJson(selectedPacks)));
                    httpPost.setEntity(new UrlEncodedFormEntity(params));

                    HttpResponse response = client.execute(httpPost);
                    this.downloadProgress = 0.1F;

                    int responseStatus = response.getStatusLine().getStatusCode();

                    if (responseStatus != 200) {
                        VTDMod.log(Level.WARN, "The download link request responded with an unexpected status code: {}", responseStatus);
                        return;
                    }

                    StringBuilder responseBody = new StringBuilder();
                    Scanner scanner = new Scanner(response.getEntity().getContent());

                    while (scanner.hasNext()) {
                        responseBody.append(scanner.nextLine());
                    }
                    this.downloadProgress = 0.35F;

                    String downloadLink = VTDMod.GSON.fromJson(responseBody.toString(), JsonObject.class).get("link").getAsString();
                    String defaultFileName = downloadLink.split("/")[downloadLink.split("/").length - 1];
                    String fileName = !this.packNameField.method_1882().equals("") ? this.packNameField.method_1882() + ".zip" : defaultFileName;

                    // Download the resource pack
                    URL url = new URL(VTDMod.BASE_URL + downloadLink);
                    URLConnection connection = url.openConnection();
                    connection.addRequestProperty("User-Agent", "VTDownloader v" + VTDMod.VERSION);
                    connection.setConnectTimeout(500);
                    connection.setConnectTimeout(4000);

                    InputStream in = connection.getInputStream();
                    Files.copy(in, new File(this.field_22787.method_1479(), fileName.trim()).toPath(), StandardCopyOption.REPLACE_EXISTING);

                    this.downloadProgress = 1.0F;
                }

                button.setSuccess(true);
            } catch (IOException e) {
                VTDMod.logError("Encountered an exception while trying to download the resource pack.", e);
                button.setSuccess(false);
            }
        });

        downloadThread.setName("VT Download");
        downloadThread.start();
    }

    protected void method_25426() {
        this.tabLeftButton = this.method_25411(new ArrowButtonWidget(40, 30, 20, 20, ArrowButtonWidget.ArrowType.LEFT, button -> {
            --this.tabIndex;
            this.updateTabButtons();
        }));
        this.tabRightButton = this.method_25411(new ArrowButtonWidget(70, 30, 20, 20, ArrowButtonWidget.ArrowType.RIGHT, button -> {
            ++this.tabIndex;
            this.updateTabButtons();
        }));

        // Reload button
        this.method_25411(new ArrowButtonWidget(10, 30, 20, 20, ArrowButtonWidget.ArrowType.CLOCKWISE, button -> {
            VTDMod.reloadRPCategories();
            this.field_22787.method_1507(new VTDScreen(this.previousScreen, this.subtitle, this.selectedPacks));
        }));
        // Done button
        this.method_25411(new class_4185(this.field_22789 - 90, this.field_22790 - 30, 80, 20, new class_2588("vtd.done"), button -> this.method_25419()));

        this.downloadButton = this.method_25411(new DownloadButtonWidget(this.field_22789 - 200, this.field_22790 - 30, 100, 20, new class_2588("vtd.download"), new class_2588("vtd.download.success"), new class_2588("vtd.download.failure"), button -> this.download((DownloadButtonWidget) button)));

        if (this.packNameField != null) {
            this.packNameField = new PackNameTextFieldWidget(this.field_22793, 10, this.field_22790 - 30, 160, 20, new class_2588("vtd.resourcePack.nameField"), this.field_22787.method_1479(), this::updateDownloadButton, TOOLTIP_SUPPLIER, this.packNameField.method_1882());
        } else {
            this.packNameField = new PackNameTextFieldWidget(this.field_22793, 10, this.field_22790 - 30, 160, 20, new class_2588("vtd.resourcePack.nameField"), this.field_22787.method_1479(), this::updateDownloadButton, TOOLTIP_SUPPLIER);
        }
        this.packNameField.method_1880(64);
        this.field_22786.add(this.packNameField);

        boolean exceptionFound = VTDMod.rpCategories.size() == 0;

        if (!exceptionFound) {
            JsonObject category = VTDMod.rpCategories.get(selectedTabIndex).getAsJsonObject();

            this.listWidget = this.method_25429(new PackListWidget(category.get("packs").getAsJsonArray(), category.get("category").getAsString()));
        } else {
            this.listWidget = this.method_25429(new PackListWidget());
        }

        this.selectedPacksListWidget = new SelectedPacksListWidget();
        this.selectedPacksListWidget.method_25333(this.field_22789 - 170);
        this.method_25429(this.selectedPacksListWidget);

        this.resetDownloadProgress();
        if (!exceptionFound) {
            this.updateTabButtons();
        } else {
            this.tabLeftButton.field_22763 = false;
            this.tabRightButton.field_22763 = false;

            this.downloadButton.field_22763 = false;
        }
    }

    public void method_25394(class_4587 matrices, int mouseX, int mouseY, float delta) {
        this.method_25434(0);
        this.listWidget.method_25394(matrices, mouseX, mouseY, delta); // Render pack list
        this.selectedPacksListWidget.method_25394(matrices, mouseX, mouseY, delta); // Render selected packs list
        this.packNameField.method_25394(matrices, mouseX, mouseY, delta); // Render pack name text field
        method_27534(matrices, this.field_22793, this.field_22785, this.field_22789 / 2, 8, 16777215); // Render title
        method_27534(matrices, this.field_22793, this.subtitle, this.field_22789 / 2, 20, 16777215); // Render subtitle

        // Render tabButtons
        for (class_4185 tabButton : this.tabButtons) {
            tabButton.method_25394(matrices, mouseX, mouseY, delta);
        }

        super.method_25394(matrices, mouseX, mouseY, delta);

        if (this.downloadProgress != -1.0F)
            this.renderDownloadProgressBar(matrices, this.field_22789 - 170, this.field_22790 - 50, this.field_22789 - 10, this.field_22790 - 40, 0.9F);
    }

    /**
     * A cursed fix to #2. Bedrockify mixins into {@link class_437#method_25434(int)}
     * and "disables" it. In order to "enable" it again, I have copied the method and, because
     * it's a different method on a different class Bedrockify won't affect it with a mixin.
     */
    @Deprecated
    @Override
    public void method_25434(int vOffset) {
        class_289 tessellator = class_289.method_1348();
        class_287 bufferBuilder = tessellator.method_1349();
        this.field_22787.method_1531().method_22813(field_22735);
        RenderSystem.color4f(1.0F, 1.0F, 1.0F, 1.0F);
        float f = 32.0F;
        bufferBuilder.method_1328(7, class_290.field_1575);
        bufferBuilder.method_22912(0.0D, this.field_22790, 0.0D).method_22913(0.0F, (float) this.field_22790 / 32.0F + (float) vOffset).method_1336(64, 64, 64, 255).method_1344();
        bufferBuilder.method_22912(this.field_22789, this.field_22790, 0.0D).method_22913((float) this.field_22789 / 32.0F, (float) this.field_22790 / 32.0F + (float) vOffset).method_1336(64, 64, 64, 255).method_1344();
        bufferBuilder.method_22912(this.field_22789, 0.0D, 0.0D).method_22913((float) this.field_22789 / 32.0F, (float) vOffset).method_1336(64, 64, 64, 255).method_1344();
        bufferBuilder.method_22912(0.0D, 0.0D, 0.0D).method_22913(0.0F, (float) vOffset).method_1336(64, 64, 64, 255).method_1344();
        tessellator.method_1350();
    }

    /**
     * Renders the download progress bar.
     * The width of it will be {@code x2 - x1}, and the height will be {@code y2 - y1}.
     * The progress of the bar will be the value of {@link #downloadProgress}
     *
     * @param matrices I don't actually know what this is for lol.
     * @param x1       the bar top left corner/start x position.
     * @param y1       the bar top left corner/start y position.
     * @param x2       the bar bottom right corner/end x position.
     * @param y2       the bar bottom right corner/end y position.
     * @param opacity  the opacity of the progress bar
     */
    @SuppressWarnings("SameParameterValue")
    private void renderDownloadProgressBar(class_4587 matrices, int x1, int y1, int x2, int y2, float opacity) {
        int progressWidth = class_3532.method_15386((float) (x2 - x1 - 4) * this.downloadProgress);
        int alpha = Math.round(opacity * 255.0F);
        int color = class_5253.class_5254.method_27764(alpha, 255, 255, 255);

        // Draw progress bar outline
        method_25294(matrices, x1 + 1, y1, x2 - 1, y1 + 1, color); // Top line
        method_25294(matrices, x1 + 1, y2, x2 - 1, y2 - 1, color); // Bottom line
        method_25294(matrices, x1, y1, x1 + 1, y2, color); // Left line
        method_25294(matrices, x2, y1, x2 - 1, y2, color); // Right line

        // Draw progress bar "progress"
        method_25294(matrices, x1 + 2, y1 + 2, x1 + progressWidth + 2, y2 - 2, color);
    }

    public void method_25393() {
        this.downloadButton.tick();
    }

    public void method_25419() {
        this.field_22787.method_1507(this.previousScreen);
    }

    private void updateTabButtons() {
        this.tabLeftButton.field_22763 = this.tabIndex > 0;
        this.tabRightButton.field_22763 = this.tabIndex < VTDMod.rpCategories.size() - 1;

        this.updateDownloadButton();

        this.tabButtons.clear();

        // Remove old buttons from this.children
        for (JsonElement category : VTDMod.rpCategories) {
            String categoryName = category.getAsJsonObject().get("category").getAsString();
            this.field_22786.removeIf(element -> element instanceof class_4185 && ((class_4185) element).method_25369().method_10851().equals(categoryName));
        }

        for (int i = 0; i < getTabNum(this.field_22789); ++i) {
            int index = i + this.tabIndex;
            if (index >= VTDMod.rpCategories.size()) break;

            JsonObject category = VTDMod.rpCategories.get(index).getAsJsonObject();
            String categoryName = category.get("category").getAsString();
            class_4185 buttonWidget = new class_4185(i * 130 + 100, 30, 120, 20, new class_2585(categoryName), button -> {
                if (this.selectedTabIndex != index) {
                    this.selectedTabIndex = index;

                    this.field_22786.remove(this.listWidget);
                    // Doesn't work as expected :/
//                    this.listWidget.replaceEntries(VTDMod.rpCategories.get(selectedTabIndex).getAsJsonObject().get("packs").getAsJsonArray());

                    JsonObject category2 = VTDMod.rpCategories.get(selectedTabIndex).getAsJsonObject();
                    this.listWidget = this.method_25429(new PackListWidget(category2.get("packs").getAsJsonArray(), category2.get("category").getAsString()));
                }
            });

            this.tabButtons.add(i, buttonWidget);
            this.field_22786.add(buttonWidget);
        }
    }

//    private static boolean isFileNameValid(String fileName) {
//        if (fileName.equals("")) {
//            return true;
//        }
//
//        // On Windows the max path length is of 260 characters. The path to
//        // the resource packs folder is normally about 50 characters long,
//        // and the path to a data packs folder is about 60 characters long.
//        // That means there are 200 characters left to use for a data/resource
//        // pack file name, but that number may vary greatly.
//        if (fileName.length() > 64) {
//            return false;
//        }
//
//        return true;
//    }

    public void updateDownloadButton() {
        this.downloadButton.field_22763 = this.selectedPacks.size() > 0 && this.packNameField.isNameValid();
    }

    public void addSelectedPack(String categoryName, String packName) {
        this.addSelectedPack(categoryName, packName, false);
    }

    public void addSelectedPack(String categoryName, String packName, boolean oneEntry) {
        if (!this.selectedPacks.containsKey(categoryName)) {
            this.selectedPacks.put(categoryName, new ArrayList<>(Collections.singleton(packName)));

            this.updateDownloadButton();
            this.selectedPacksListWidget.updateEntries();
        } else if (!this.isPackSelected(categoryName, packName)) {
            List<String> packs = oneEntry ? new ArrayList<>() : this.selectedPacks.get(categoryName);
            packs.add(packName);
            this.selectedPacks.replace(categoryName, packs);

            this.updateDownloadButton();
            this.selectedPacksListWidget.updateEntries();
        }
    }

    public boolean isPackSelected(String categoryName, String packName) {
        if (!this.selectedPacks.containsKey(categoryName)) return false;

        return this.selectedPacks.get(categoryName).contains(packName);
    }

    public void removeSelectedPack(String categoryName, String packName) {
        if (!isPackSelected(categoryName, packName)) return;

        this.selectedPacks.get(categoryName).remove(packName);

        if (this.selectedPacks.get(categoryName).size() == 0) {
            this.selectedPacks.remove(categoryName);
        }

        this.updateDownloadButton();
        this.selectedPacksListWidget.updateEntries();
    }

    public class_310 getClient() {
        return this.field_22787;
    }

    public class_327 getTextRenderer() {
        return this.field_22793;
    }

    /**
     * Sets {@link #downloadProgress} back to the default.
     */
    public void resetDownloadProgress() {
        this.downloadProgress = -1.0F;
    }

    public String getSelectedCategory() {
        return this.listWidget.categoryName;
    }
}
